@@ -16,8 +16,11 @@
 package org.gradle.api.internal;
 
 import groovy.lang.Closure;
-import groovy.lang.MissingPropertyException;
-import org.gradle.api.*;
+import org.gradle.api.InvalidUserDataException;
+import org.gradle.api.NamedDomainObjectCollection;
+import org.gradle.api.Namer;
+import org.gradle.api.Rule;
+import org.gradle.api.UnknownDomainObjectException;
 import org.gradle.api.internal.collections.CollectionEventRegister;
 import org.gradle.api.internal.collections.CollectionFilter;
 import org.gradle.api.internal.plugins.DefaultConvention;
@@ -25,10 +28,26 @@ import org.gradle.api.plugins.Convention;
 import org.gradle.api.plugins.ExtensionContainer;
 import org.gradle.api.specs.Spec;
 import org.gradle.api.specs.Specs;
+import org.gradle.internal.metaobject.AbstractDynamicObject;
+import org.gradle.internal.metaobject.BeanDynamicObject;
+import org.gradle.internal.metaobject.DynamicObject;
+import org.gradle.internal.metaobject.GetPropertyResult;
+import org.gradle.internal.metaobject.InvokeMethodResult;
+import org.gradle.internal.metaobject.MixInClosurePropertiesAsMethodsDynamicObject;
 import org.gradle.internal.reflect.Instantiator;
 import org.gradle.util.ConfigureUtil;
 
-import java.util.*;
+import java.util.ArrayList;
+import java.util.Collection;
+import java.util.Collections;
+import java.util.HashSet;
+import java.util.List;
+import java.util.Map;
+import java.util.NavigableMap;
+import java.util.Set;
+import java.util.SortedMap;
+import java.util.SortedSet;
+import java.util.TreeMap;
 
 public class DefaultNamedDomainObjectCollection<T> extends DefaultDomainObjectCollection<T> implements NamedDomainObjectCollection<T>, DynamicObjectAware {
 
@@ -150,7 +169,7 @@ public class DefaultNamedDomainObjectCollection<T> extends DefaultDomainObjectCo
     }
 
     public String getDisplayName() {
-        return String.format("%s container", getTypeDisplayName());
+        return getTypeDisplayName() + " container";
     }
 
     public SortedMap<String, T> getAsMap() {
@@ -294,20 +313,20 @@ public class DefaultNamedDomainObjectCollection<T> extends DefaultDomainObjectCo
         return getType().getSimpleName();
     }
 
-    private class ContainerDynamicObject extends CompositeDynamicObject {
+    private class ContainerDynamicObject extends MixInClosurePropertiesAsMethodsDynamicObject {
         private ContainerDynamicObject(ContainerElementsDynamicObject elementsDynamicObject) {
             setObjects(new BeanDynamicObject(DefaultNamedDomainObjectCollection.this), elementsDynamicObject, convention.getExtensionsAsDynamicObject());
         }
 
         @Override
-        protected String getDisplayName() {
+        public String getDisplayName() {
             return DefaultNamedDomainObjectCollection.this.getDisplayName();
         }
     }
 
     private class ContainerElementsDynamicObject extends AbstractDynamicObject {
         @Override
-        protected String getDisplayName() {
+        public String getDisplayName() {
             return DefaultNamedDomainObjectCollection.this.getDisplayName();
         }
 
@@ -317,12 +336,11 @@ public class DefaultNamedDomainObjectCollection<T> extends DefaultDomainObjectCo
         }
 
         @Override
-        public T getProperty(String name) throws MissingPropertyException {
+        public void getProperty(String name, GetPropertyResult result) {
             T t = findByName(name);
-            if (t == null) {
-                return (T) super.getProperty(name);
+            if (t != null) {
+                result.result(t);
             }
-            return t;
         }
 
         @Override
@@ -336,11 +354,9 @@ public class DefaultNamedDomainObjectCollection<T> extends DefaultDomainObjectCo
         }
 
         @Override
-        public Object invokeMethod(String name, Object... arguments) throws groovy.lang.MissingMethodException {
+        public void invokeMethod(String name, InvokeMethodResult result, Object... arguments) {
             if (isConfigureMethod(name, arguments)) {
-                return ConfigureUtil.configure((Closure) arguments[0], getByName(name));
-            } else {
-                return super.invokeMethod(name, arguments);
+                result.result(ConfigureUtil.configure((Closure) arguments[0], getByName(name)));
             }
         }
 
@@ -349,7 +365,7 @@ public class DefaultNamedDomainObjectCollection<T> extends DefaultDomainObjectCo
         }
     }
 
-    protected static interface Index<T> {
+    protected interface Index<T> {
         void put(String name, T value);
 
         T get(String name);
