@@ -18,14 +18,22 @@ package org.gradle.api.internal.file.copy;
 import com.google.common.collect.ImmutableList;
 import groovy.lang.Closure;
 import org.gradle.api.Action;
+import org.gradle.api.InvalidUserDataException;
 import org.gradle.api.NonExtensible;
-import org.gradle.api.file.*;
+import org.gradle.api.Transformer;
+import org.gradle.api.file.CopyProcessingSpec;
+import org.gradle.api.file.CopySpec;
+import org.gradle.api.file.DuplicatesStrategy;
+import org.gradle.api.file.FileCopyDetails;
+import org.gradle.api.file.FileTree;
+import org.gradle.api.file.FileTreeElement;
+import org.gradle.api.file.RelativePath;
 import org.gradle.api.internal.ChainingTransformer;
 import org.gradle.api.internal.ClosureBackedAction;
 import org.gradle.api.internal.file.FileResolver;
 import org.gradle.api.internal.file.pattern.PatternMatcherFactory;
-import org.gradle.api.specs.NotSpec;
 import org.gradle.api.specs.Spec;
+import org.gradle.api.specs.Specs;
 import org.gradle.api.tasks.util.PatternSet;
 import org.gradle.internal.reflect.Instantiator;
 import org.gradle.internal.typeconversion.NotationParser;
@@ -33,12 +41,18 @@ import org.gradle.util.ConfigureUtil;
 
 import java.io.File;
 import java.io.FilterReader;
-import java.util.*;
+import java.nio.charset.Charset;
+import java.util.ArrayList;
+import java.util.Collection;
+import java.util.LinkedHashSet;
+import java.util.List;
+import java.util.Map;
+import java.util.Set;
 import java.util.regex.Pattern;
 
 @NonExtensible
 public class DefaultCopySpec implements CopySpecInternal {
-    private static final NotationParser<Object, String> PATH_NOTATION_PARSER = PathNotationParser.create();
+    private static final NotationParser<Object, String> PATH_NOTATION_PARSER = PathNotationConverter.parser();
     protected final FileResolver fileResolver;
     private final Set<Object> sourcePaths;
     private Object destDir;
@@ -51,13 +65,15 @@ public class DefaultCopySpec implements CopySpecInternal {
     private Boolean caseSensitive;
     private Boolean includeEmptyDirs;
     private DuplicatesStrategy duplicatesStrategy;
+    private String filteringCharset;
+
 
     public DefaultCopySpec(FileResolver resolver, Instantiator instantiator) {
         this.fileResolver = resolver;
         this.instantiator = instantiator;
         sourcePaths = new LinkedHashSet<Object>();
         childSpecs = new ArrayList<CopySpecInternal>();
-        patternSet = new PatternSet();
+        patternSet = resolver.getPatternSetFactory().create();
         duplicatesStrategy = null;
     }
 
@@ -86,14 +102,20 @@ public class DefaultCopySpec implements CopySpecInternal {
         return this;
     }
 
-    public CopySpec from(Object sourcePath, Closure c) {
-        if (c == null) {
+    public CopySpec from(Object sourcePath, final Closure c) {
+        return from(sourcePath, new ClosureBackedAction<CopySpec>(c));
+    }
+
+    public CopySpec from(Object sourcePath, Action<? super CopySpec> configureAction) {
+        if (configureAction == null) {
             from(sourcePath);
             return this;
         } else {
             CopySpecInternal child = addChild();
             child.from(sourcePath);
-            return ConfigureUtil.configure(c, instantiator.newInstance(CopySpecWrapper.class, child));
+            CopySpecWrapper wrapper = instantiator.newInstance(CopySpecWrapper.class, child);
+            configureAction.execute(wrapper);
+            return wrapper;
         }
     }
 
@@ -129,13 +151,19 @@ public class DefaultCopySpec implements CopySpecInternal {
     }
 
     public CopySpec into(Object destPath, Closure configureClosure) {
-        if (configureClosure == null) {
+        return into(destPath, new ClosureBackedAction<CopySpec>(configureClosure));
+    }
+
+    public CopySpec into(Object destPath, Action<? super CopySpec> copySpec) {
+        if (copySpec == null) {
             into(destPath);
             return this;
         } else {
             CopySpecInternal child = addChild();
             child.into(destPath);
-            return ConfigureUtil.configure(configureClosure, instantiator.newInstance(CopySpecWrapper.class, child));
+            CopySpecWrapper wrapper = instantiator.newInstance(CopySpecWrapper.class, child);
+            copySpec.execute(wrapper);
+            return wrapper;
         }
     }
 
@@ -166,13 +194,12 @@ public class DefaultCopySpec implements CopySpecInternal {
     public CopySpec filesMatching(String pattern, Action<? super FileCopyDetails> action) {
         Spec<RelativePath> matcher = PatternMatcherFactory.getPatternMatcher(true, isCaseSensitive(), pattern);
         return eachFile(
-                new MatchingCopyAction(matcher, action));
+            new MatchingCopyAction(matcher, action));
     }
 
     public CopySpec filesNotMatching(String pattern, Action<? super FileCopyDetails> action) {
         Spec<RelativePath> matcher = PatternMatcherFactory.getPatternMatcher(true, isCaseSensitive(), pattern);
-        return eachFile(
-                new MatchingCopyAction(new NotSpec<RelativePath>(matcher), action));
+        return eachFile(new MatchingCopyAction(Specs.<RelativePath>negate(matcher), action));
     }
 
     public CopySpec include(String... includes) {
@@ -253,9 +280,13 @@ public class DefaultCopySpec implements CopySpecInternal {
     }
 
     public CopySpec filter(final Closure closure) {
+        return filter(new ClosureBackedTransformer(closure));
+    }
+
+    public CopySpec filter(final Transformer<String, String> transformer) {
         copyActions.add(new Action<FileCopyDetails>() {
             public void execute(FileCopyDetails fileCopyDetails) {
-                fileCopyDetails.filter(closure);
+                fileCopyDetails.filter(transformer);
             }
         });
         return this;
@@ -280,8 +311,12 @@ public class DefaultCopySpec implements CopySpecInternal {
     }
 
     public CopySpec rename(Closure closure) {
+        return rename(new ClosureBackedTransformer(closure));
+    }
+
+    public CopySpec rename(Transformer<String, String> renamer) {
         ChainingTransformer<String> transformer = new ChainingTransformer<String>(String.class);
-        transformer.add(closure);
+        transformer.add(renamer);
         copyActions.add(new RenamingCopyAction(transformer));
         return this;
     }
@@ -310,7 +345,7 @@ public class DefaultCopySpec implements CopySpecInternal {
     }
 
     public CopySpec eachFile(Closure closure) {
-        copyActions.add(new ClosureBackedAction<FileCopyDetails>(closure));
+        copyActions.add(ConfigureUtil.configureUsing(closure));
         return this;
     }
 
@@ -342,6 +377,20 @@ public class DefaultCopySpec implements CopySpecInternal {
         return this.new DefaultCopySpecResolver(null);
     }
 
+    public String getFilteringCharset() {
+        return buildRootResolver().getFilteringCharset();
+    }
+
+    public void setFilteringCharset(String charset) {
+        if (charset == null) {
+            throw new InvalidUserDataException("filteringCharset must not be null");
+        }
+        if (!Charset.isSupported(charset)) {
+            throw new InvalidUserDataException(String.format("filteringCharset %s is not supported by your JVM", charset));
+        }
+        this.filteringCharset = charset;
+    }
+
     public class DefaultCopySpecResolver implements CopySpecResolver {
 
         private CopySpecResolver parentResolver;
@@ -485,7 +534,7 @@ public class DefaultCopySpec implements CopySpecInternal {
         }
 
         public PatternSet getPatternSet() {
-            PatternSet patterns = new PatternSet();
+            PatternSet patterns = fileResolver.getPatternSetFactory().create();
             patterns.setCaseSensitive(isCaseSensitive());
             patterns.include(this.getAllIncludes());
             patterns.includeSpecs(getAllIncludeSpecs());
@@ -500,6 +549,18 @@ public class DefaultCopySpec implements CopySpecInternal {
                 child.buildResolverRelativeToParent(this).walk(action);
             }
         }
+
+        public String getFilteringCharset() {
+            if (filteringCharset != null) {
+                return filteringCharset;
+            }
+            if (parentResolver != null) {
+                return parentResolver.getFilteringCharset();
+            }
+            return Charset.defaultCharset().name();
+        }
+
+
     }
 
 }
