@@ -18,9 +18,12 @@ package org.gradle.tooling.internal.connection;
 
 import com.google.common.collect.Lists;
 import org.gradle.internal.composite.GradleParticipantBuild;
-import org.gradle.tooling.*;
+import org.gradle.tooling.BuildActionExecuter;
+import org.gradle.tooling.BuildController;
+import org.gradle.tooling.GradleConnectionException;
+import org.gradle.tooling.ModelBuilder;
+import org.gradle.tooling.ProjectConnection;
 import org.gradle.tooling.connection.ModelResult;
-import org.gradle.tooling.model.ProjectIdentifier;
 import org.gradle.tooling.internal.adapter.ProtocolToModelAdapter;
 import org.gradle.tooling.internal.consumer.converters.BuildInvocationsConverter;
 import org.gradle.tooling.internal.consumer.converters.FixedBuildIdentifierProvider;
@@ -28,6 +31,7 @@ import org.gradle.tooling.internal.consumer.parameters.ConsumerOperationParamete
 import org.gradle.tooling.model.GradleProject;
 import org.gradle.tooling.model.HasGradleProject;
 import org.gradle.tooling.model.HierarchicalElement;
+import org.gradle.tooling.model.ProjectIdentifier;
 import org.gradle.tooling.model.build.BuildEnvironment;
 import org.gradle.tooling.model.gradle.BasicGradleProject;
 import org.gradle.tooling.model.gradle.BuildInvocations;
@@ -67,8 +71,7 @@ public class ToolingClientCompositeModelBuilder<T> {
                 final List<ModelResult<T>> participantResults = buildResultsForParticipant(participantConnector);
                 results.addAll(participantResults);
             } catch (GradleConnectionException e) {
-                String message = String.format("Could not fetch models of type '%s' using client-side composite connection.", modelType.getSimpleName());
-                results.add(new DefaultFailedModelResult<T>(participantConnector.toBuildIdentifier(), new GradleConnectionException(message, e)));
+                results.add(new DefaultFailedModelResult<T>(participantConnector.toBuildIdentifier(), e));
             }
         }
         return results;
@@ -174,24 +177,13 @@ public class ToolingClientCompositeModelBuilder<T> {
                 util.configureRequest(actionExecuter);
                 Map<String, T> actionResults = actionExecuter.run();
                 for (final String projectPath : actionResults.keySet()) {
-                    T identified = unpackAndTransform(participant.toProjectIdentifier(projectPath), actionResults.get(projectPath));
-                    ModelResult<T> result = createModelResult(identified);
+                    ModelResult<T> result = createModelResult(actionResults.get(projectPath));
                     results.add(result);
                 }
             } finally {
                 projectConnection.close();
             }
         }
-
-        protected T unpackAndTransform(ProjectIdentifier projectIdentifier, T t) {
-            Object sourceObject = protocolToModelAdapter.unpack(t);
-            // TODO:DAZ This should be done in the BuildControllerAdapter, then we wouldn't need to adapt here
-            return transform(projectIdentifier, sourceObject);
-        }
-
-        protected T transform(ProjectIdentifier projectIdentifier, Object sourceObject) {
-            return protocolToModelAdapter.adapt(modelType, sourceObject, new FixedBuildIdentifierProvider(projectIdentifier));
-        }
     }
 
     /**
@@ -238,6 +230,10 @@ public class ToolingClientCompositeModelBuilder<T> {
                 constructBuildInvocationsFromGradleProject(participant, childProject, results);
             }
         }
+
+        private T transform(ProjectIdentifier projectIdentifier, Object sourceObject) {
+            return new FixedBuildIdentifierProvider(projectIdentifier).applyTo(protocolToModelAdapter.builder(modelType)).build(sourceObject);
+        }
     }
 
     private static final class FetchPerProjectModelAction<V> implements org.gradle.tooling.BuildAction<Map<String, V>> {
