@@ -15,21 +15,40 @@
  */
 package org.gradle.tooling.model.eclipse;
 
+import org.gradle.api.Incubating;
 import org.gradle.tooling.model.ProjectDependency;
 
 /**
  * Represents a dependency on another Eclipse project.
  */
-public interface EclipseProjectDependency extends ProjectDependency {
+public interface EclipseProjectDependency extends ProjectDependency, EclipseClasspathEntry {
     /**
     * Returns the target of this dependency.
     *
-    * @return The target project. Does not return null.
+    * @return The target project, or null for a dependency on a different build within a composite.
     */
+    @Deprecated
     HierarchicalEclipseProject getTargetProject();
 
+    /**
+     * Returns the identifier of the target eclipse project.
+     *
+     * @return The target identifier, never null.
+     * @since 2.14
+     */
+    @Incubating
+    EclipseProjectIdentifier getTarget();
+
     /**
      * Returns the path to use for this project dependency.
      */
     String getPath();
+
+    /**
+     * Marks this dependency as exported.
+     *
+     * @return whether this dependency needs to be exported.
+     * @since 2.5
+     */
+    boolean isExported();
 }
