@@ -689,7 +689,7 @@ public class GrailsClassUtils {
      * @return true if the method is a property getter
      */
     public static boolean isPropertyGetter(Method method) {
-        return !Modifier.isStatic(method.getModifiers()) && Modifier.isPublic(method.getModifiers()) && isGetter(method.getName(), method.getParameterTypes());
+        return !Modifier.isStatic(method.getModifiers()) && Modifier.isPublic(method.getModifiers()) && isGetter(method.getName(), method.getReturnType(), method.getParameterTypes());
     }
 
     /**
@@ -728,8 +728,22 @@ public class GrailsClassUtils {
      * @param name The name of the method
      * @param args The arguments
      * @return true if it is a javabean property getter
+     * @deprecated use {@link #isGetter(String, Class, Class[])} instead because this method has a defect for "is.." method with Boolean return types.
      */
     public static boolean isGetter(String name, Class<?>[] args) {
+        return isGetter(name, boolean.class, args);
+    }
+
+    /**
+     * Returns true if the name of the method specified and the number of arguments make it a javabean property getter.
+     * The name is assumed to be a valid Java method name, that is not verified.
+     *
+     * @param name The name of the method
+     * @param returnType The return type of the method
+     * @param args The arguments
+     * @return true if it is a javabean property getter
+     */
+    public static boolean isGetter(String name, Class returnType, Class<?>[] args) {
         if (!StringUtils.hasText(name) || args == null)return false;
         if (args.length != 0)return false;
 
@@ -737,7 +751,7 @@ public class GrailsClassUtils {
             name = name.substring(3);
             if (isPropertyMethodSuffix(name)) return true;
         }
-        else if (name.startsWith("is")) {
+        else if (name.startsWith("is") && returnType == boolean.class) {
             name = name.substring(2);
             if (isPropertyMethodSuffix(name)) return true;
         }
@@ -797,15 +811,28 @@ public class GrailsClassUtils {
      *
      * @param getterName The getter name
      * @return The property name equivalent
+     * @deprecated Use {@link #getPropertyForGetter(String, Class)} instead because this method has a defect for "is.." method with Boolean return types.
      */
     public static String getPropertyForGetter(String getterName) {
+        return getPropertyForGetter(getterName, boolean.class);
+    }
+
+    /**
+     * Returns a property name equivalent for the given getter name and return type or null if it is not a valid getter. If not null
+     * or empty the getter name is assumed to be a valid identifier.
+     *
+     * @param getterName The getter name
+     * @param returnType The type the method returns
+     * @return The property name equivalent
+     */
+    public static String getPropertyForGetter(String getterName, Class returnType) {
         if (getterName == null || getterName.length() == 0) return null;
 
         if (getterName.startsWith("get")) {
             String prop = getterName.substring(3);
             return convertValidPropertyMethodSuffix(prop);
         }
-        if (getterName.startsWith("is")) {
+        if (getterName.startsWith("is") && returnType == boolean.class) {
             String prop = getterName.substring(2);
             return convertValidPropertyMethodSuffix(prop);
         }
@@ -836,6 +863,9 @@ public class GrailsClassUtils {
         if (Character.isUpperCase(suffix.charAt(0))) {
             return Character.toLowerCase(suffix.charAt(0)) + suffix.substring(1);
         }
+        if('_' == suffix.charAt(0)) {
+            return suffix;
+        }
         return null;
     }
 
